--
-- Most of the code adapted from
--  - oGlow by haste
--  - SpecialEvents-Bags-2.0 by Tekkub et.al.
--

local _G = getfenv(0)

local bags = {}
local hide = {}

local function hideBorder(self)
	local t = self.wutznuw
	if not t or not bags[t.bag][t.slot][3] then return end
	t:Hide()
	bags[t.bag][t.slot][3] = nil
end

local addon = CreateFrame("Frame")
do
	local total = 0
	addon:SetScript("OnUpdate", function(self, elapsed)
		if not next(hide) then return end
		total = total + elapsed
		if total < 5 then return end
		local t = GetTime()
		for frame, time in pairs(hide) do
			if t > time then
				hideBorder(frame)
				hide[frame] = nil
			end
		end
		total = 0
	end)
end
addon:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)
function addon:PLAYER_ENTERING_WORLD()
	for bag = 0, 4 do
		bags[bag] = {}
		for slot = 1, GetContainerNumSlots(bag) do
			local itemlink = GetContainerItemLink(bag, slot)
			local _, stack = GetContainerItemInfo(bag, slot)
			bags[bag][slot] = {itemlink, stack}
		end
	end
	self:RegisterEvent("BAG_UPDATE")
end
function addon:PLAYER_LEAVING_WORLD()
	self:UnregisterEvent("BAG_UPDATE")
end
addon:RegisterEvent("PLAYER_ENTERING_WORLD")
addon:RegisterEvent("PLAYER_LEAVING_WORLD")

function addon:BAG_UPDATE(bag)
	if not bag then return end
	for slot = 1, GetContainerNumSlots(bag) do
		if not bags[bag] then bags[bag] = {} end
		if not bags[bag][slot] then bags[bag][slot] = {} end
		local current = bags[bag][slot]
		local itemlink = GetContainerItemLink(bag, slot)
		local _, stack = GetContainerItemInfo(bag, slot)
		if current[1] ~= itemlink or current[2] ~= stack then
			current[1], current[2], current[3] = itemlink, stack, true
		end
	end
end

local function createBorder(frame)
	local border = frame:CreateTexture(nil, "OVERLAY")
	border:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	border:SetBlendMode("ADD")
	border:SetAlpha(1)
	border:SetWidth(70)
	border:SetHeight(70)
	border:SetPoint("CENTER", frame)
	border:SetVertexColor(1, 1, 1)
	frame.wutznuw = border
end

hooksecurefunc("ContainerFrame_OnShow", function(self)
	local bag = self:GetID()
	if not bags[bag] then return end
	for i, v in pairs(bags[bag]) do
		if v[3] then
			local item = self.size - i + 1
			local f = _G[self:GetName() .. "Item" .. item]
			if not f.wutznuw then createBorder(f) end
			f.wutznuw.bag = bag
			f.wutznuw.slot = i
			f.wutznuw:Show()
		end
	end
end)

hooksecurefunc("ContainerFrame_OnHide", function(self)
	local bag = self:GetID()
	if not bags[bag] then return end
	for i, v in pairs(bags[bag]) do
		if v[3] then
			local item = self.size - i + 1
			local f = _G[self:GetName() .. "Item" .. item]
			if f.wutznuw then
				hide[f] = GetTime() + 5
			end
		end
	end
end)

local hideFuncs = {
	"ContainerFrameItemButton_OnClick",
	"ContainerFrameItemButton_OnModifiedClick",
	--"ContainerFrameItemButton_OnEnter",
	"ContainerFrameItemButton_OnDragStart",
	"ContainerFrameItemButton_OnDragEnd",
}
for i, v in ipairs(hideFuncs) do
	hooksecurefunc(v, hideBorder)
end

